# -*- coding: utf-8 -*-

"""
This is a small pyramid extension that allows to add rules for redirecting
the PATH_INFO portion of a requested URL.

Usage example:
    def main(global_config, **settings):
        config = Configurator(settings=settings)
        config.include('pyramid_redirect')
        # add url redirecting rules...
        #   first parameter is a regular expression
        #   second parameter is the target url
        config.add_redirect_rule(r'/favicon.ico', r'/static/favicon.ico')
        config.add_redirect_rule(r'/gallery/(?P<subpath>.*)',
                                r'/root/%(subpath)s')
        #
        # ... rest of configuration
        #
        # return WSGI application instance
        return config.make_wsgi_app()
"""

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp

from pyramid.httpexceptions import HTTPFound
from pyramid.httpexceptions import HTTPMovedPermanently
import logging
import re


__version__ = 0.3


# Add configuration directive
def includeme(config):
    config.add_directive('add_redirect_rule', add_redirect_rule)
    config.add_tween('pyramid_redirect.redirect_tween_factory')


# Configuration directive for adding a redirect rule
def add_redirect_rule(config, pattern, target, permanent=False):
    tpattern = pattern
    if not pattern.startswith(r'^'):
        tpattern = '^' + tpattern
    if not pattern.endswith(r'$'):
        tpattern = tpattern + r'$'
    cpattern = re.compile(tpattern)
    if not hasattr(config.registry, 'redirect_rules'):
        config.registry.redirect_rules = []
    config.registry.redirect_rules.append((pattern, cpattern, target, permanent))

# Tween to perform URL redirecting before a request is handled by Pyramid
def redirect_tween_factory(handler, registry):

    if not hasattr(registry, 'redirect_rules'):
        return handler

    def redirect_tween(request):
        for pattern, cpattern, target, permanent in request.registry.redirect_rules:
            url = request.url
            if request.registry.settings.get('pyramid_redirect.structlog'):
                import structlog
                logger = structlog.getLogger(__name__)
                logger.debug('Matching Pattern', pattern=pattern, url=url)
            else:
                logger = logging.getLogger(__name__)
                logger.debug('Matching pattern "%s" against "%s" ' \
                    % (pattern, url))
            mo = cpattern.match(url)
            if mo is not None:
                url = target % mo.groupdict()
                if request.registry.settings.get('pyramid_redirect.structlog'):
                    import structlog
                    logger = structlog.getLogger(__name__)
                    logger.info('URL Redirected', from_=request.url, to=url)
                else:
                    logger = logging.getLogger(__name__)
                    logger.info('Redirecting url: %s --> %s' \
                        % (request.url, url))
                if permanent:
                    return HTTPMovedPermanently(url)
                else:
                    return HTTPFound(url)
        return handler(request)

    return redirect_tween



# -*- coding: utf-8 -*-

# This software is distributed under the FreeBSD License.
# See the accompanying file LICENSE for details.
#
# Copyright 2012 Benjamin Hepp

import unittest

from pyramid import testing


class TestRedirectTween(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()
        self.config.include('pyramid_redirect')

    def tearDown(self):
        testing.tearDown()

    def _makeOne(self):
        from pyramid_redirect import redirect_tween_factory
        return redirect_tween_factory(dummy_handler, self.config.registry)

    def _callFUT(self, request):
        tween = self._makeOne()
        return tween(request)

    def test_redirect_no_rules(self):
        request = testing.DummyRequest(url='http://example.com/abc')
        self._callFUT(request)
        self.assertEqual(request.path_info, r'/')

    def test_redirect_rule1(self):
        self.config.add_redirect_rule(
            r'http://example\.com/abc/qwe(?P<num>[0-9]+)/(?P<op>[a-zA-Z]+)',
            'http://example.com/%(op)s/%(num)s',
        )
        request = testing.DummyRequest(url='http://example.com/abc/qwe15/get')
        result = self._callFUT(request)
        self.assertEqual(result.status_code, 302)
        self.assertEqual(result.location, r'http://example.com/get/15')

    def test_redirect_rule2(self):
        self.config.add_redirect_rule(
            r'http://example\.com/(?P<user>[a-zA-Z0-9_]+)/(?P<what>[a-zA-Z0-9_]+)/(?P<op>[a-zA-Z]+)',  # noqa
            'http://example.com/%(op)s(%(user)s.%(what)s)',
            permanent=False,
        )
        request = testing.DummyRequest(url='http://example.com/root/foo/get')
        result = self._callFUT(request)
        self.assertEqual(result.status_code, 302)
        self.assertEqual(result.location, r'http://example.com/get(root.foo)')

    def test_redirect_rule3(self):
        self.config.add_redirect_rule(
            r'http://example\.com/favicon\.ico',
            'http://example.com/static/favicon.ico',
            permanent=True
        )
        request = testing.DummyRequest(url='http://example.com/favicon.ico')
        result = self._callFUT(request)
        self.assertEqual(result.status_code, 301)
        self.assertEqual(result.location, r'http://example.com/static/favicon.ico')  # noqa

    def test_redirect_rule4(self):
        self.config.add_redirect_rule(
            r'http://example\.com/favicon\.ico',
            'http://example.com/static/favicon.ico')
        request = testing.DummyRequest(url='http://example.com/favicon.icon')
        result = self._callFUT(request)
        self.assertIsNone(result)  # no redirect


def dummy_handler(request):
    return None



import os

# this is a hack to prevent an ugly bug when
# setuptools performs sys.modules restoration
import multiprocessing

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
readme = open(os.path.join(here, 'README.rst')).read()
changes = open(os.path.join(here, 'CHANGES.rst')).read()


requires=[
    'pyramid>=1.3a6',
]

setup(
    name='pyramid_redirect',
    version='0.3',
    description='Small Pyramid extension for redirecting urls',
    long_description=readme + '\n' + changes,
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    packages=['pyramid_redirect'],
    install_requires=requires,
    author='NiteoWeb Ltd.',
    author_email='info@niteoweb.com',
    license='BSD',
    url='https://github.com/niteoweb/pyramid_redirect',
    keywords='pyramid redirect pylons web',
    tests_require=requires,
    test_suite='pyramid_redirect',
)
